import io
import os
import subprocess
import tempfile

import jinja2

# =============================================================================
def generate_outfile(
        templatesdir,
        templatename,
        context,
        outfilename,
        log = None,
        cachedir = None,
        ):

    # Template rendering
    # -------------------------------------------------------------------------
    log.info("Generating SVG using '{}' template".format(templatename))
    log.debug("Context : {}".format(context))

    log.debug(
            "Initializing jinja2 with template dir '{}'" \
                    .format(templatesdir)
                    )
    jj2_env = jinja2.Environment(
            loader = jinja2.FileSystemLoader(
                str(templatesdir),
                )
            )

    try:
        jj2_tpl = jj2_env.get_template(
                # Jinja specific path format
                "{}/template.svg.j2".format(templatename),
                )
    except Exception as e:
        log.error("Could not find template '{}'".format(templatename))
        log.debug(e, exc_info=1)
        return None

    # To SVG
    # -------------------------------------------------------------------------
    if outfilename.suffix == ".svg":
        jj2_tpl.stream(context).dump( str(outfilename) )
        return outfilename

    # To PNG with inkscape
    # -------------------------------------------------------------------------
    if outfilename.suffix == ".png":

        cachedir.mkdir(parents=True, exist_ok=True)

        tmpsvg = tempfile.NamedTemporaryFile(
                suffix=".svg",
                mode = "w",
                delete = False,
                dir = str(cachedir),
                )
        tmpsvg.close()
        try:
            log.info(
                    "Exporting to {} using inkscape" \
                            .format(outfilename.suffix),
                            )
            import subprocess

            jj2_tpl.stream(context).dump( tmpsvg.name )

            inkscape_process = subprocess.Popen(
                    [
                        "inkscape",
                        tmpsvg.name,
                        "--export-filename",
                        str(outfilename),
                        ],
                    stdout = subprocess.PIPE,
                    stderr = subprocess.STDOUT,
                    universal_newlines = True,
                    )

            for line_out in iter(inkscape_process.stdout.readline, ""):
                log.debug(line_out)

            inkscape_process.stdout.close()

            rv = inkscape_process.wait()

            if rv != 0:
                raise Exception(
                        "Bad inkscape return code '{}'" \
                                .format(inkscape_process.returncode)
                                )


            return outfilename

        except Exception as e:
            log.warning("Failed to export with inkscape")
            log.debug(e, exc_info=True)

        finally:
            os.unlink(tmpsvg.name)

    # To png, pdf or ps with cairosvg
    # -------------------------------------------------------------------------
    if outfilename.suffix in [ ".png", ".pdf", ".ps" ]:

        log.info("Exporting to {} using cairosvg".format(outfilename.suffix))

        try:
            import cairosvg
        except ImportError as e:
            log.error(
                    "Failed to export to '{}' with cairosvg" \
                            .format(
                                outfilename,
                                )
                            )
            log.debug(e)

        else:
            svg_str = jj2_tpl.render(context)

            if outfilename.suffix == ".png":
                conversion_fun = cairosvg.svg2png
            elif outfilename.suffix == ".pdf":
                conversion_fun = cairosvg.svg2pdf
            elif outfilename.suffix == ".ps":
                conversion_fun = cairosvg.svg2ps

            conversion_fun(
                bytestring = svg_str,
                write_to = str(outfilename),
                )

            return outfilename

    # To unsupported format
    # -------------------------------------------------------------------------
    log.error(
            "Can't export to '{}' : unsupported format '{}'" \
                    .format(
                        outfilename,
                        outfilename.suffix,
                        )
                    )
    return None

# =============================================================================
def generate_pic(
        templatesdir,
        templatename,
        context,
        outform="svg",
        log = None,
        cachedir = None,
        ):

    if outform not in ["svg", "png"]:
        raise ValueError("unsupported output format '{}'".format(outform))

    # Template rendering
    # -------------------------------------------------------------------------
    log.info("Generating SVG using '{}' template".format(templatename))
    log.debug("Context : {}".format(context))

    log.debug(
            "Initializing jinja2 with template dir '{}'" \
                    .format(templatesdir)
                    )
    jj2_env = jinja2.Environment(
            loader = jinja2.FileSystemLoader(
                str(templatesdir),
                )
            )

    try:
        jj2_tpl = jj2_env.get_template(
                # Jinja specific path format
                "{}/template.svg.j2".format(templatename),
                )
    except jinja2.exceptions.TemplateNotFound as e:
        raise e
    except Exception as e:
        log.error("Could not find template '{}'".format(templatename))
        log.debug(e, exc_info=1)
        return None

    # To SVG
    # -------------------------------------------------------------------------
    if outform == "svg":
        return jj2_tpl.render(context)

    # To PNG with inkscape
    # -------------------------------------------------------------------------
    if outform == "png":

        if cachedir is None:
            cachedir_tmp = tempfile.TemporaryDirectory()
            cachedir = cachedir_tmp.name
        else:
            cachedir.mkdir(parents=True, exist_ok=True)

        tmpsvg = tempfile.NamedTemporaryFile(
                suffix=".svg",
                mode = "w",
                delete = False,
                dir = str(cachedir),
                )
        tmpsvg.close()
        tmppng = tempfile.NamedTemporaryFile(
                suffix = ".png",
                mode = "w",
                delete = False,
                dir = str(cachedir),
                )
        tmppng.close()
        try:
            log.info(
                    "Exporting to {} using inkscape" \
                            .format(outform),
                            )
            import subprocess

            jj2_tpl.stream(context).dump( tmpsvg.name )

            inkscape_process = subprocess.Popen(
                    [
                        "inkscape",
                        tmpsvg.name,
                        "--export-filename",
                        tmppng.name,
                        ],
                    stdout = subprocess.PIPE,
                    stderr = subprocess.STDOUT,
                    universal_newlines = True,
                    )

            for line_out in iter(inkscape_process.stdout.readline, ""):
                log.debug(line_out)

            inkscape_process.stdout.close()

            rv = inkscape_process.wait()

            if rv != 0:
                raise Exception(
                        "Bad inkscape return code '{}'" \
                                .format(inkscape_process.returncode)
                                )

            with open(tmppng.name, "rb") as fh:
                buf = io.BytesIO(fh.read())

            return buf

        except Exception as e:
            log.warning("Failed to export with inkscape")
            log.debug(e, exc_info=True)

        finally:
            os.unlink(tmpsvg.name)
            os.unlink(tmppng.name)

            try:
                cachedir_tmp.cleanup()
            except:
                pass

        return None



# -------------------------------------------------------------------------
class SSBUCharacter:
    """Infos needed about an SSBU character"""
    def __init__(
            self,
            name,
            codename,
            spritersurls,
            smashggid = None,
            ):
        self.spritersurls = spritersurls
        self.smashggid = smashggid
        self.name = name
        self.codename = codename
        
# -------------------------------------------------------------------------
EVERYONE = [
        SSBUCharacter(
            smashggid = 1530,
            name = "banjo & kazooie",
            codename = "buddy",
            spritersurls = [
                "https://www.spriters-resource.com/download/121027/",
                ],
            ),
        SSBUCharacter(
            smashggid = None,
            name = "bayonetta",
            codename = "bayonetta",
            spritersurls = [
                "https://www.spriters-resource.com/download/111299/",
                "https://www.spriters-resource.com/download/111300/",
                ],
            ),
        SSBUCharacter(
            smashggid = 1272,
            name = "bowser jr",
            codename = "koopajr",
            spritersurls = [
                "https://www.spriters-resource.com/download/111303/",
                ],
            ),
        SSBUCharacter(
            smashggid = 1273,
            name = "bowser",
            codename = "koopa",
            spritersurls = [
                "https://www.spriters-resource.com/download/111302/",
                ],
            ),
        SSBUCharacter(
            smashggid = 1539,
            name = "byleth",
            codename = "master",
            spritersurls = [
                "https://www.spriters-resource.com/download/125348/",
                ],
            ),
        SSBUCharacter(
            smashggid = 1274,
            name = "captain falcon",
            codename = "captain",
            spritersurls = [
                "https://www.spriters-resource.com/download/111304/"
                ],
            ),
        SSBUCharacter(
                smashggid = None,
                name = "charizard",
                codename = "plizardon",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111351/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1409,
                name = "chrom",
                codename = "chrom",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111305/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1275,
                name = "cloud",
                codename = "cloud",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111306/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "corrin",
                codename = "kamui",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111307/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1277,
                name = "daisy",
                codename = "daisy",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111308/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "dark pit",
                codename = "pitb",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111309/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1279,
                name = "diddy kong",
                codename = "diddy",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111311/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1280,
                name = "donkey kong",
                codename = "donkey",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111312/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1282,
                name = "dr mario",
                codename = "mariod",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111313/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1283,
                name = "duck hunt",
                codename = "duckhunt",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111314/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1285,
                name = "falco",
                codename = "falco",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111315/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1286,
                name = "fox",
                codename = "fox",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111316/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1287,
                name = "ganondorf",
                codename = "ganon",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111317/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "greninja",
                codename = "gekkouga",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111318/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1526,
                name = "hero",
                codename = "brave",
                spritersurls = [
                    "https://www.spriters-resource.com/download/119842/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1290,
                name = "ice climbers",
                codename = "ice_climber",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111319/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1291,
                name = "ike",
                codename = "ike",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111320/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1292,
                name = "inkling",
                codename = "inkling",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111322/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "ivysaur",
                codename = "pfushigisou",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111352/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1293,
                name = "jigglypuff",
                codename = "purin",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111324/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1410,
                name = "ken",
                codename = "ken",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111325/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1294,
                name = "king dedede",
                codename = "dedede",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111326/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1295,
                name = "kirby",
                codename = "kirby",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111328/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1296,
                name = "link",
                codename = "link",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111329/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1297,
                name = "little mac",
                codename = "littlemac",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111330/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1298,
                name = "lucario",
                codename = "lucario",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111331/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1299,
                name = "lucas",
                codename = "lucas",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111332/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1300,
                name = "lucina",
                codename = "lucina",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111333/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1301,
                name = "luigi",
                codename = "luigi",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111334/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1302,
                name = "mario",
                codename = "mario",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111335/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "marth",
                codename = "marth",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111336/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1305,
                name = "mega man",
                codename = "rockman",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111337/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1307,
                name = "meta knight",
                codename = "metaknight",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111338/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1310,
                name = "mewtwo",
                codename = "mewtwo",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111339/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "mii fighter",
                codename = "miifighter",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111340/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1415,
                name = "mii gunner",
                codename = "miigunner",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111340/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "mii swordsman",
                codename = "miiswordsman",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111340/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1313,
                name = "ness",
                codename = "ness",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111342/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1314,
                name = "olimar",
                codename = "pikmin",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111343/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1315,
                name = "pacman",
                codename = "pacman",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111344/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1316,
                name = "palutena",
                codename = "palutena",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111345/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1317,
                name = "peach",
                codename = "peach",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111346/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1318,
                name = "pichu",
                codename = "pichu",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111347/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1319,
                name = "pikachu",
                codename = "pikachu",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111348/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1320,
                name = "pit",
                codename = "pit",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111349/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1321,
                name = "pokemon trainer",
                codename = "ptrainer",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111350/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1412,
                name = "richter",
                codename = "richter",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111354/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1322,
                name = "ridley",
                codename = "ridley",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111355/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1323,
                name = "rob",
                codename = "robot",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111356/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1324,
                name = "robin",
                codename = "reflet",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111357/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1325,
                name = "rosalina and luma",
                codename = "rosetta",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111358/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1326,
                name = "roy",
                codename = "roy",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111359/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1327,
                name = "ryu",
                codename = "ryu",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111360/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1328,
                name = "samus",
                codename = "samus",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111361/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1329,
                name = "sheik",
                codename = "sheik",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111362/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1330,
                name = "shulk",
                codename = "shulk",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111363/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "simon",
                codename = "simon",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111364/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1331,
                name = "snake",
                codename = "snake",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111365/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1332,
                name = "sonic",
                codename = "sonic",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111366/",
                    ],
                ),
        SSBUCharacter(
                smashggid = None,
                name = "squirtle",
                codename = "pzenigame",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111353/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1333,
                name = "toon link",
                codename = "toonlink",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111367/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1334,
                name = "villager",
                codename = "murabito",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111368/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1335,
                name = "wario",
                codename = "wario",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111369/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1336,
                name = "wii fit trainer",
                codename = "wiifit",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111370/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1337,
                name = "wolf",
                codename = "wolf",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111371/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1338,
                name = "yoshi",
                codename = "yoshi",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111372/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1339,
                name = "young link",
                codename = "younglink",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111373/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1340,
                name = "zelda",
                codename = "zelda",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111374/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1341,
                name = "zero suit samus",
                codename = "szerosuit",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111375/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1405,
                name = "mr game and watch",
                codename = "gamewatch",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111341/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1406,
                name = "incineroar",
                codename = "gaogaen",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111321/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1407,
                name = "king k rool",
                codename = "krool",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111327/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1408,
                name = "dark samus",
                codename = "samusd",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111310/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1413,
                name = "isabelle",
                codename = "shizue",
                spritersurls = [
                    "https://www.spriters-resource.com/download/111323/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1441,
                name = "piranha plant",
                codename = "packun",
                spritersurls = [
                    "https://www.spriters-resource.com/download/113440/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1453,
                name = "joker",
                codename = "jack",
                spritersurls = [
                    "https://www.spriters-resource.com/download/116168/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1532,
                name = "terry",
                codename = "dolly",
                spritersurls = [
                    "https://www.spriters-resource.com/download/123089/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1747,
                name = "min min",
                codename = "tantan",
                spritersurls = [
                    "https://www.spriters-resource.com/download/134242/",
                    ],
                ),
        SSBUCharacter(
                smashggid = 1766,
                name = "steve",
                codename = "pickel",
                spritersurls = [
                    "https://www.spriters-resource.com/download/140939/",
                    ],
                ),
        ]




import json
import os, os.path
import pathlib

import requests

from . import characters_ssbu

# =============================================================================
API_HOST = "api.smash.gg"
API_ENDPOINT = "gql/alpha"
API_SCHEME = "https"
API_URL = "{scheme}://{host}/{endpoint}".format(
        scheme = API_SCHEME,
        host = API_HOST,
        endpoint = API_ENDPOINT,
        )

CHARACTERS = { c.smashggid : c.name for c in characters_ssbu.EVERYONE }

GET_PLAYERDATA = lambda tag: {"tag": tag,}

# =============================================================================
# -----------------------------------------------------------------------------
class Player():
    """A Player, as registered by the smash.gg API, and their characters
    choices and placement in a tournament"""

    # -------------------------------------------------------------------------
    def __init__(
            self,
            id,
            prefix,
            gamerTag,
            placement,
            seeding,
            twitterHandle = None,
            chars = None
            ):
        self.id            = int(id) # actually intended to store an Entrant id
        self.prefix        = ( "" if prefix is None else prefix )
        self.gamerTag      = gamerTag
        self.placement     = int(placement)
        self.seeding       = int(seeding)
        self.twitterHandle = ( "" if twitterHandle is None else twitterHandle )

        try:
            self.playerdata = GET_PLAYERDATA(self.gamerTag)
        except:
            self.playerdata = {}

        # Check if there's a redirect in the players DB
        try:
            self.gamerTag = self.playerdata["tag"]
        except:
            pass

        # Prefix is superseeded by sponsors infos in DB
        try:
            self.prefix = self.playerdata["sponsor"]
        except:
            pass

        if chars is None:
            self.chars = {}
        else:
            self.chars = chars

    # -------------------------------------------------------------------------
    def add_character_selection(self, character, win):

        if type(character) != tuple:
            try:
                charname = CHARACTERS[character]
            except KeyError:
                charname = character # Unknown char -> sgg id

            try:
                skin = self.playerdata["skins"][charname]
            except KeyError:
                skin = "00" # default skin

            character = ( charname, skin )


        try:
            self.chars[character] += ( 1.01 if win else 1.00 )
            # This 1.01 / 1.00 tricks should hold until the player loses 101
            # matches with one character and wins 100 matches with another...
            # during one tournament.
        except KeyError:
            self.chars[character] = ( 1.01 if win else 1.00 )

    # -------------------------------------------------------------------------
    def get_mains(self):
        return [ cv[0] for cv in sorted(
            self.chars.items(),
            key = lambda cv: cv[1],
            reverse = True,
            ) ]

    # -------------------------------------------------------------------------
    def conf(self):

        # The char list looks like 'character1_skin1 (12.08), character2_skin2
        # (3.02)' where the number between parenthesis is the number of time
        # the character was played
        charslist = ", ".join( [ "{}_{} ({:.2f})".format(c,s,n) for (c,s),n in
            sorted( self.chars.items(), key = lambda cv: cv[1], reverse =
                True,) ])

        return """
[player {tag}]
tag: {tag}
team: {pfx}
seeding: {seed}
placement: {plc}
twitter: {twi}
characters: {charslist}""" \
            .format(
                    tag = self.gamerTag,
                    pfx = self.prefix,
                    seed = self.seeding,
                    plc = self.placement,
                    twi = self.twitterHandle,
                    charslist  = charslist,
                    )


    # -------------------------------------------------------------------------
    def __str__(self):
        # The char list looks like 'character1_skin1 (12.08), character2_skin2
        # (3.02)' where the number between parenthesis is the number of time
        # the character was played
        charslist = ", ".join( [ "{}_{} ({:.2f})".format(c,s,n) for (c,s),n in
            sorted( self.chars.items(), key = lambda cv: cv[1], reverse =
                True,) ])
        return "{plc}/ [{pfx}] {tag} -- {charslist}" \
                .format(
                    tag = self.gamerTag,
                    pfx = self.prefix,
                    plc = self.placement,
                    charslist  = charslist,
                        )

# -----------------------------------------------------------------------------
class Tournament():
    """A Tournament, as registered by the smash.gg API"""

    # -------------------------------------------------------------------------
    def __init__(
            self,
            id,
            name,
            slug,
            startAt,
            numEntrants,
            venueAddress = None,
            venueName = None,
            city = None,
            countryCode = None,
            hashtag = None,
            ):
        self.id           = id
        self.name         = name
        self.slug         = slug.split("/")[-1]
        self.startAt      = startAt
        self.numEntrants  = numEntrants
        self.venueAddress = venueAddress
        self.venueName    = venueName
        self.city         = city
        self.countryCode  = countryCode
        self.hashtag      = hashtag

    # -------------------------------------------------------------------------
    def conf(self):
        return """
[Tournament]
name: {name}
date: {date}
location: {loc} - {addr} ({city}, {ctry})
slug: {slug}
numEntrants: {nbe}""" \
            .format(
                    id = self.id,
                    name = self.name,
                    date = self.startAt,
                    loc = self.venueName,
                    addr = self.venueAddress,
                    city = self.city,
                    ctry = self.countryCode,
                    slug = self.slug,
                    nbe = self.numEntrants,
                    )

    # -------------------------------------------------------------------------
    def clean_name(self, name_seo_delimiter):

        if name_seo_delimiter is not None and name_seo_delimiter != "":
            self.name = self.name.split(name_seo_delimiter)[0].strip()

        return self

    # -------------------------------------------------------------------------
    def __str__(self):
        return "Tournament {name} ({date} at {loc}), {nbe} entrants" \
                .format(
                    id = self.id,
                    name = self.name,
                    date = self.startAt,
                    loc = self.venueName,
                    addr = self.venueAddress,
                    city = self.city,
                    ctry = self.countryCode,
                    slug = self.slug,
                    nbe = self.numEntrants,
                    )



# =============================================================================
def run_query(
        query_name,
        variables = {},
        token = "",
        proxy = None,
        query_dir = pathlib.Path("queries"),
        query_extension = "gql",
        api_url = API_URL,
        log = None,
        ):

    # Load query
    query_path = query_dir / "{}.{}".format(
            query_name,
            query_extension,
            )
    query  = ""

    with query_path.open("r") as query_file:
        query = query_file.read()

    # Build payload
    payload = {
            "query": query,
            "variables": json.dumps(variables),
            }

    # Build headers
    headers = {
            "Authorization": "Bearer {token}".format(token = token),
            "Accept": "application/json",
            "Content-Type": "application/json"
            }

    # Send the query
    try:
        log.debug("Sending query '{}' with variables:".format(query_name))
        log.debug(json.dumps(variables))
    except AttributeError:
        pass

    rv = requests.post(
            API_URL,
            json.dumps(payload).encode("utf-8"),
            headers = headers,
            proxies = {"http": proxy, "https": proxy},
            )

    try:
        rv_json = rv.json()
    except Exception as e:
        log.error("HTTP request failed")
        log.error(e)
        log.debug(e, exc_info=True)
        log.debug(rv)

    # Try to return the data, or log the error and return None
    try:
        log.debug("query complexity : {}" \
                .format(
                    rv_json["extensions"]["queryComplexity"],
                    ))
        return rv_json["data"]
    except KeyError:
        try:
            log.error("GraphQL error")
            log.error(rv_json)
        except AttributeError:
            pass
        return None




NAME = "lokrez"
DESCRIPTION = "Smash.gg-connected top8 graphics generator for Super Smash " \
              "Bros Ultimate tournaments"

URL = "https://git.lertsenem.com/lertsenem/ssbu_lokrez"
AUTHOR = "Lertsenem"
ENTITY = "Smash@Lyon"
AUTHOR_EMAIL = "lertsenem@lertsenem.com"
ENTITY_EMAIL = "dev@smashatlyon.com"

VERSION_MAJOR = 2
VERSION_MINOR = 1
VERSION_PATCH = 0

__version__ = "{}.{}.{}".format(
        VERSION_MAJOR,
        VERSION_MINOR,
        VERSION_PATCH,
        )

VERSION_NAME = "{} - {}".format(
        NAME,
        __version__,
        )

__license__ = "MIT"



import json
import os, os.path
import pathlib

import requests

from . import characters_ssbu

# =============================================================================
API_HOST = "api.challonge.com"
API_ENDPOINT = "v1"
API_SCHEME = "https"
API_URL = "{scheme}://{host}/{endpoint}".format(
        scheme = API_SCHEME,
        host = API_HOST,
        endpoint = API_ENDPOINT,
        )

CHARACTERS = { c.smashggid : c.name for c in characters_ssbu.EVERYONE }

PLAYERSKINS = {}


# =============================================================================
def get_participants( api_key, tournament, proxy = None, log = None ):
    
    payload = {
            "api_key": api_key,
            }
    headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
            }

    rv = requests.get(
            "{url}/{tournament}/participants.json".format(
                url = API_URL,
                tournament = tournament,
                )
            json.dumps(payload).encode("utf-8"),
            headers = headers,
            proxies = { "http": proxy, "https": proxy},
            )

    try:
        rv_json = rv.json()
    except Exception as e:
        log.error("HTTP request failed")
        log.error(e)
        log.debug(e, exc_info=True)
        log.debug(rv)

    return rv_json



import io
import pathlib
import sys
import zipfile

import requests

from .characters_ssbu import EVERYONE

# -----------------------------------------------------------------------------
def download_file(url, with_progressbar = True, proxy = None):

    r = requests.get(
            url,
            stream = with_progressbar,
            proxies = { "https": proxy, "http": proxy },
            )

    if not with_progressbar:
        return io.BytesIO(r.content)

    total = r.headers.get("content-length")

    f = io.BytesIO()

    if total is not None:
        downloaded = 0
        total = int(total)
        for data in r.iter_content(
                chunk_size = max(int(total/1000), 1024*1024),
                ):
            f.write(data)
            downloaded += len(data)
            done = int(50*downloaded/total)
            sys.stdout.write( "\r[{}{}] ({:02d}%)".format(
                "█" * done,
                " " * (50-done),
                done*2,
                ) )
            sys.stdout.flush()
        sys.stdout.write("\n")
    else:
        f = write(r.content)

    return f

# -----------------------------------------------------------------------------
def download_res_ssbu(dstdir, proxy = None, log=None):
    """Downloads SSBU resources from spriters and rename them according to
    lokrez expectations"""

    stock_icons_url = "https://www.spriters-resource.com/download/111395/"

    # -------------------------------------------------------------------------
    # Download stock icons
    log.warning("Downloading stock icons...")
    fstocks = download_file(stock_icons_url, proxy = proxy)
    zfstocks = zipfile.ZipFile(fstocks)

    # Iter over each character
    for character in EVERYONE:
        log.warning("Downloading images for {}...".format(character.name))

        # Create directory for this character
        chardir = dstdir / character.name

        try:
            chardir.mkdir()

        except FileExistsError:
            log.info(
                    "Directory already exists for {}".format(character.name)
                    )
                
            try:
                next(chardir.iterdir())
                log.warning(
                        "Directory not empty for {}, skipping" \
                                .format(character.name)
                        )
                continue
            except StopIteration:
                log.info(
                        "Directory empty, downloading",
                        )

        # Download urls & write image files
        for url in character.spritersurls:
            try:
                f = download_file(url, proxy = proxy)
            except Exception as e:
                try:
                    log.warning("Download failed ({}), retrying".format(e))
                    f = download_file(url, proxy = proxy)
                except Exception as e:
                    log.error("Download failed({})".format(e))
                    log.debug(e, exc_info = True)
                    continue

            with zipfile.ZipFile(f) as zfchar:
                for zf in [zfchar,zfstocks]:
                    for source_filename in zf.namelist():

                        if "No Gamma Fix" in source_filename:
                            continue

                        if character.codename not in source_filename:
                            continue

                        target_filename = pathlib.Path(source_filename).name

                        if target_filename in ["","Tag.txt"]:
                            continue

                        target_filename = pathlib.Path(source_filename).name

                        target_filename = target_filename.replace(
                                character.codename,
                                character.name,
                                )

                        log.debug("Writing file '{}'".format(target_filename))

                        target_filename = chardir / target_filename

                        with open(str(target_filename), "wb") as tf:
                            tf.write(zf.read(source_filename))

# =============================================================================
if __name__ == '__main__':

    import argparse
    import logging
    import tempfile

    logging.basicConfig(
            level = logging.DEBUG,
            format = "%(message)s",
            )

    parser = argparse.ArgumentParser()

    parser.add_argument(
            "dstdir",
            default = None,
            help = "directory where to store the downloaded resources " \
                   "(default to a temporary file)",
                   )

    args = parser.parse_args()

    if args.dstdir is None:
        args.dstdir = tempfile.mkdtemp()
        logging.warning(
                "Storing in temporary directory : {}".format(args.dstdir)
                )

    download_res_ssbu(
            dstdir = args.dstdir,
            log = logging,
            )



import distutils, distutils.util
import os
import setuptools
import sys

import lokrez, lokrez.version

# Load README for long_description
with open("README.adoc", "r") as fh:
    long_description = fh.read()

# Windows build
if os.name == "nt":
    import cx_Freeze

    arch = distutils.util.get_platform().split("-")[-1]

    build_exe_options = {
            "packages": ["os"],
            "include_msvcr": True,
            "include_files": [ "lokrez/data", "lokrez/templates", "lokrez/queries" ],
            "includes": ["idna.idnadata"], # Because of cx-freeze bug
            }

    bdist_msi_options = {
            "upgrade_code": "{123456789-1337-8483-ABCD-DEADBEEFCAFE}",
            "add_to_path": True,
            "initial_target_dir": r"[LocalAppDataFolder]\{}\{}" \
                    .format(
                        lokrez.version.ENTITY,
                        lokrez.version.NAME,
                        ),
            }

    base = None

    cx_Freeze.setup(
            name = lokrez.version.NAME,
            version = lokrez.version.__version__.replace("dev", "1337"),
            author = lokrez.version.AUTHOR,
            author_email = lokrez.version.AUTHOR_EMAIL,
            description = lokrez.version.DESCRIPTION,
            install_requires = [
                "jinja2",
                "requests",
                "appdirs",
                ],
            options = {
                "build_exe": build_exe_options,
                "bdist_msi": bdist_msi_options,
                },

            executables = [
                    cx_Freeze.Executable(
                        script = "lokrez/__main__.py",
                        targetName = "{}.exe".format(lokrez.version.NAME),
                        base = base,
                        ),
                    ],
            )

    sys.exit(0)

# Linux build
if os.name == "posix":

    setuptools.setup(
            name = lokrez.version.NAME,
            version = lokrez.__version__,
            author = lokrez.version.AUTHOR,
            author_email = lokrez.version.AUTHOR_EMAIL,
            description = lokrez.version.DESCRIPTION,
            long_description = long_description,
            long_description_content_type = "text/plain",
            url = lokrez.version.URL,
            packages = ["lokrez"],
            install_requires = [
                "jinja2",
                "requests",
                "appdirs",
                ],
            license = lokrez.version.__license__,
            classifiers = [
                "Programming Language :: Python :: 3",
                "License :: OSI Approved :: MIT License",
                "Operating System :: OS Independent",
                "Intended Audience :: End Users/Desktop",
                "Topic :: Multimedia :: Graphics",
                "Environment :: Console"
                ],
            python_requires = ">=3.5",
            keywords = "smash ultimate local results esport smashgg",
#        data_files = ("res", ["res/playerskinsdb.json"])
            package_data = {
                "lokrez": [
                    "data/*.json",
                    "queries/*.gql",
                    "templates/*/*.j2",
                    ],
                },
            entry_points = {
                "console_scripts": [
                    "lokrez = lokrez:main",
                    ],
                },
            )



import sys

# Add module parent directory to path if needed
if __package__ is None and not hasattr(sys, "frozen"):
    import pathlib

    path = pathlib.Path(__file__).absolute()
    sys.path.insert(0, str(path.parent.parent))


import lokrez

if __name__ == '__main__':
    lokrez.main()



import argparse
import configparser
import datetime
import html
import json
import logging
import pathlib
import requests
import sys
import urllib

import appdirs

from . import export
from . import resources
from . import smashgg
from . import version

# =============================================================================
__version__ = version.__version__
__license__ = version.__license__

ROOTDIR = pathlib.Path(__file__).absolute().parent

APPDIRS = appdirs.AppDirs(version.NAME, version.ENTITY)

LOG_DUMMY = logging.getLogger("dummy")
LOG_DUMMY.addHandler(logging.NullHandler())

DEFAULT_DIR_TEMPLATES = ROOTDIR / "templates"

# =============================================================================
def get_templates_list(
        dir_templates = DEFAULT_DIR_TEMPLATES,
        ):

    templates_list = []

    dir_templates_path = pathlib.Path(dir_templates)

    for potential_template in dir_templates_path.iterdir():

        if (potential_template / "template.svg.j2").is_file():
            templates_list.append(potential_template.name)

    return templates_list

# =============================================================================
def get_infos_from_url(
        url,
        token,
        options = {},
        outform = "dict",
        top = 8,
        proxy = None,
        log = LOG_DUMMY,
        ):

    url_parsed = urllib.parse.urlparse(url)

    if url_parsed.netloc not in [ "smash.gg" ]:
        raise ValueError("Unsupported domain name")
    if outform not in [ "dict", "lkrz" ]:
        raise ValueError("Unsupported outform")

    # -------------------------------------------------------------------------
    if url_parsed.netloc == "smash.gg":

        if (url_parsed.path.split("/")[1] != "tournament"):
            raise Exception("No tournament found in url {}".format(url_parsed.path.split("/")))

        # Get infos from smash.gg and write the config file
        tournament, top_players = getTournamentTop(
                id_or_slug = url_parsed.path.split("/")[2],
                get_prefixes = options.get("use_smashgg_prefixes", False),
                top = top,
                token = token,
                proxy = proxy,
                log = log,
                )

        if tournament is None or top_players is None:
            log.error("Could not load data from smash.gg")
            raise Exception("Could not load data from smash.gg")

    # -------------------------------------------------------------------------
    if outform == "dict":
        return {
                "tournament": tournament,
                "players": top_players,
                }

    # -------------------------------------------------------------------------
    if outform == "lkrz":
        return "\n".join(
                [ tournament.conf() ] \
                        + list(map(
                            lambda p:p.conf(),
                            top_players,
                            ))
                        )

# =============================================================================
def generate_pic(
        infos_or_lkrzfile = None,
        template = None,
        outform = "svg",
        options = {},
        dir_templates = DEFAULT_DIR_TEMPLATES,
        dir_res = None,
        dir_cache = None,
        log = LOG_DUMMY,
        ):

    if outform not in ["svg", "png"]:
        raise Exception("Unsupported outform")

    if type(infos_or_lkrzfile) == str:
        # TODO : load lkrz as dict infos
        raise NotImplementedError()
    else:
        infos = infos_or_lkrzfile

    # -------------------------------------------------------------------------
    # Build the context which will be passed to the template
    context = {
            "tournament": infos["tournament"].clean_name(
                options.get(
                    "name_seo_delimiter",
                    None
                    )
                ),
            "players" : sorted(
                infos["players"],
                key = lambda p: p.placement,
                ),
            "dir_res_ssbu": dir_res,
            "dir_template": str(dir_templates/template),
            "options": options.get("template_options", []),
            }

    pic = export.generate_pic(
            dir_templates,
            template,
            context,
            outform,
            log = log,
            cachedir = dir_cache,
            )

    if pic is None:
        raise Exception("Failed to generate pic")

    return pic

# =============================================================================
def main():

    # -------------------------------------------------------------------------
    parser = argparse.ArgumentParser(
            formatter_class = argparse.ArgumentDefaultsHelpFormatter,
            )

    subparsers = parser.add_subparsers(
            dest = "command",
            help = "commands",
            )

    parser.add_argument(
            "--proxy", "-p",
            default = None,
            help = "the proxy to use",
            )

    # -------------------------------------------------------------------------
    init_parser = subparsers.add_parser(
            "init",
            formatter_class = argparse.ArgumentDefaultsHelpFormatter,
            )

    init_parser.add_argument(
            "game",
            default = "ssbu",
            help = "The game you want to initialize the resources for",
            )

    init_parser.add_argument(
            "--imgdir", "-ID",
            type = pathlib.Path,
            default = pathlib.Path(APPDIRS.user_data_dir) / "res",
            help = "The directory we should download the resources to",
            )

    # -------------------------------------------------------------------------
    top8_parser = subparsers.add_parser(
            "top8",
            formatter_class = argparse.ArgumentDefaultsHelpFormatter,
            )

    top8_parser.add_argument(
            "tournament",
            default = None,
            help = "The tournament slug or id",
            )

    top8_parser.add_argument(
            "--token", "-t",
            default = None,
            help = "the authentication token to use",
            )

    top8_parser.add_argument(
            "--imgdir", "-ID",
            type = pathlib.Path,
            default = pathlib.Path(APPDIRS.user_data_dir) / "res",
            help = "The directories containing images, be careful whether " \
                   "you specify an absolute path or a relative one.",
            )
    top8_parser.add_argument(
            "--playerskinsdb", "-PD",
            type = (lambda s: s if s.startswith("http") else pathlib.Path(s)),
            default = ROOTDIR / "data" / "playerskinsdb.json",
            help = "A JSON file path or url matching player tags, characters,"\
                   " sponsors, and preferred skins",
            )
    top8_parser.add_argument(
            "--cachedir", "-CD",
            type = pathlib.Path,
            default = pathlib.Path(APPDIRS.user_cache_dir),
            help = "A directory to use for temporary files",
            )
    top8_parser.add_argument(
            "--templatesdir", "-TD",
            type = pathlib.Path,
            default = DEFAULT_DIR_TEMPLATES,
            help = "The local result templates directory",
            )

    top8_parser.add_argument(
            "--template", "-T",
            default = "rebootlyon2020",
            help = "The local result template to use",
            )
    top8_parser.add_argument(
            "--template-options", "-O",
            action = "append",
            default = [],
            help = "Template-specific options",
            )

    top8_parser.add_argument(
            "--lkrz-file", "-f",
            type = pathlib.Path,
            default = None,
            help = "The lkrz file in which the results are stored ; if it " \
                   "does not exist, one will be created from the smashgg data",
            )
    top8_parser.add_argument(
            "--outfile", "-o",
            type = pathlib.Path,
            default = None,
            help = "The SVG or PNG local result file to output to ; if it's " \
                   "not specified, it will use the tournament slug as name",
            )

    top8_parser.add_argument(
            "--name-seo-delimiter",
            default = None,
            help = "A character that will delimit in a tournament name what " \
                   "really is its name, and what's actually here for SEO "    \
                   "purposes (example: in 'Cornismash #42 - Ultimate Weekly " \
                   "Lyon', only the 'Cornismash #42' is the tournament name, "\
                   "the rest is here to help find the tournament).",
            )
    top8_parser.add_argument(
            "--use-smashgg-prefixes", "-P",
            action = "store_true",
            help = "Use the prefixes (sponsor, team, etc) set by players on " \
                   "smash.gg for the tournament",
                   )

    # -------------------------------------------------------------------------
    parser.add_argument( "--verbose", "-v",
                         default = 0,
                         action = "count",
                         help = "increase verbosity" )

    parser.add_argument( "--version", "-V",
                         default = False,
                         action = "store_true",
                         help = "show version number" )

    # -------------------------------------------------------------------------
    args = parser.parse_args()

    # Set log level
    # -------------------------------------------------------------------------
    log = logging.getLogger(version.NAME)
    log.setLevel(logging.DEBUG)

    log_handler_console = logging.StreamHandler()
    log_handler_console.setLevel(logging.WARNING)

    if(args.verbose >= 2):
        log_handler_console.setLevel(logging.DEBUG)
    elif(args.verbose >=1):
        log_handler_console.setLevel(logging.INFO)
    else:
        log_handler_console.setLevel(logging.WARNING)

    log_formatter_console = logging.Formatter("%(name)s:%(levelname)s: %(message)s")

    log_handler_console.setFormatter(log_formatter_console)

    log.addHandler(log_handler_console)

    # Print version if required
    # -------------------------------------------------------------------------
    if args.version:
        print(version.VERSION_NAME)
        return 0

    # -------------------------------------------------------------------------
    if args.command not in [ "init", "top8" ]:
        parser.print_help()
        return 1

    # -------------------------------------------------------------------------
    if args.command == "init":
        args.imgdir.mkdir(parents=True, exist_ok=True)
        resources.download_res_ssbu(
                dstdir = args.imgdir,
                proxy = args.proxy,
                log = log,
                )
        return 0
    # -------------------------------------------------------------------------
    if args.command == "top8":

        # Initialize PLAYERSKINS db
        log.debug("loading playerskins db from '{}'" \
                .format(args.playerskinsdb))
        try:
            PLAYERSKINS =  requests.get(args.playerskinsdb).json()
            smashgg.GET_PLAYERDATA = (lambda tag: PLAYERSKINS[tag.lower()])
        except:
            with args.playerskinsdb.open("r", encoding="utf8") as f:
                PLAYERSKINS = json.load(f)
                smashgg.GET_PLAYERDATA = (lambda tag: PLAYERSKINS[tag.lower()])

        #
        tournament = None
        top_players = {}

        if args.lkrz_file is not None and args.lkrz_file.exists():
            lkrz = configparser.ConfigParser()
            lkrz.read(str(args.lkrz_file))

            log.info("Loading data from '{}'".format(args.lkrz_file))

            for s in lkrz:
                section = lkrz[s]
                if s == "Tournament":
                    tournament = smashgg.Tournament(
                            id = 0,
                            name = section["name"],
                            slug = section["slug"],
                            startAt = datetime.datetime.strptime(
                                section["date"],
                                "%Y-%m-%d %H:%M:%S",
                                ),
                            numEntrants = int(section["numEntrants"]),
                            venueName = section["location"],
                            )

                elif s.startswith("player "):
                    chars = {}
                    for char in section["characters"].split(","):
                        c = char.strip()
                        charname = c.split("_")[0]
                        charskin = c.split("_")[1].split(" ")[0]
                        charscore = float(c.split("(")[1].split(")")[0])

                        chars[(charname,charskin)] = charscore

                    player = smashgg.Player(
                            id = 0,
                            prefix = section["team"],
                            gamerTag = section["tag"],
                            placement = section["placement"],
                            seeding = section["seeding"],
                            twitterHandle = section["twitter"],
                            chars = chars,
                            )

                    top_players[player.gamerTag] = player

        else:

            # Get infos from smash.gg and write the config file
            tournament, top_players = getTournamentTop(
                    id_or_slug = args.tournament,
                    get_prefixes = args.use_smashgg_prefixes,
                    top = 8,
                    token = args.token,
                    proxy = args.proxy,
                    log = log,
                    )

            if tournament is None or top_players is None:
                log.error("Could not load data from smash.gg")
                return 1

            lkrz_data = "\n".join(
                    [ tournament.conf() ] \
                    + list(map(
                        lambda p:p.conf(),
                        top_players,
                        ))
                    )

            if args.lkrz_file is None:
                args.lkrz_file = pathlib.Path(
                        "{}.lkrz".format(tournament.slug)
                        )

            with args.lkrz_file.open("w", encoding="utf8") as f:
                f.write(lkrz_data)

        # Default outfile is 'tournament-slug.svg'
        if args.outfile is None:
            args.outfile = pathlib.Path(
                    "{}.svg".format(tournament.slug),
                    )

        # Build the context which will be passed to the template
        try:
            dir_res_ssbu = args.imgdir.as_uri() # not absolute => error
        except ValueError:
            dir_res_ssbu = args.imgdir.as_posix()

        context = {
                "tournament": tournament.clean_name(args.name_seo_delimiter),
                "players" : sorted(
                    top_players,
                    key = lambda p: p.placement,
                    ),
                "dir_res_ssbu": dir_res_ssbu,
                "dir_template": str(args.templatesdir / args.template),
                "options": args.template_options,
                }

        rv = export.generate_outfile(
                args.templatesdir,
                args.template,
                context,
                args.outfile,
                log = log,
                cachedir = args.cachedir,
                )

        if rv is None:
            return 1

        log.info("Successfully saved outfile as '{}'".format(rv))

        return 0

# -----------------------------------------------------------------------------
def getTournamentTop(
        id_or_slug,
        get_prefixes = True,
        top=8,
        token = "",
        proxy = None,
        log=LOG_DUMMY):
    """Returns a tuple : the smashgg.Tournament object and a list of the top
    smashgg.Player in that tournament."""

    # TODO if url matches challonge
    #
    #data = challonge.get_participants(
    #        api_key = token,
    #        tournament = id_or_slug,
    #        )
    #
    #top_array = []*top
    #for p in data:
    #   top_array[p["participant"]["final_rank"]] = ...

    # -------------------------------------------------------------------------
    # Select the right event (the one with the most entrants or the most sets)
    def selectBiggestEvent(data, log=LOG_DUMMY):

        try:
            event = data["events"][0]
        except:
            log.error("No event found in data")
            log.debug(data)
            return None

        try:
            numEntrants = event["numEntrants"]
        except KeyError:
            numEntrants = event["standings"]["pageInfo"]["total"]

        for e in data["events"]:
            try:
                ne = e["numEntrants"]
            except KeyError:
                ne = e["standings"]["pageInfo"]["total"]

            if ne > numEntrants:
                event = e
                numEntrants = ne

        log.info("Selected Event '{}' with {} entrants" \
                .format(
                    event["name"],
                    numEntrants,
                    ))

        return event

    # -------------------------------------------------------------------------
    data = None

    try:
        data = smashgg.run_query(
                query_name = "getTournamentTopById",
                variables = {
                    "id" : int(id_or_slug), # If this fails, it's a slug
                    "top": top,
                    },
                query_dir = ROOTDIR / "queries",
                token = token,
                proxy = proxy,
                log = log,
                )

    except ValueError:
        data = smashgg.run_query(
                query_name = "getTournamentTopBySlug",
                variables = {
                    "slug" : id_or_slug,
                    "top": top,
                    },
                query_dir = ROOTDIR / "queries",
                token = token,
                proxy = proxy,
                log = log,
                )

    try:
        tournament_data = data["tournament"]
    except:
        log.error("Failed to load Tournaments")
        return None,None

    if tournament_data is None:
        log.error("Failed to load Tournament")
        return None,None

    event = selectBiggestEvent(tournament_data, log)

    if event is None :
        return None,None

    # Get the tournament
    tournament = smashgg.Tournament(
            id = tournament_data["id"],
            slug = tournament_data["slug"],
            name = tournament_data["name"],
            startAt = \
                    datetime.datetime. \
                    fromtimestamp(tournament_data["startAt"]),
            numEntrants = event["standings"]["pageInfo"]["total"],
            venueAddress = tournament_data["venueAddress"],
            venueName = tournament_data["venueName"],
            city = tournament_data["city"],
            countryCode = tournament_data["countryCode"],
            hashtag = tournament_data["hashtag"],
            )


    # Get the top players
    top_players = {}

    standings = event["standings"]["nodes"]

    for standing in standings :

        seeding = None
        for seed in standing["entrant"]["seeds"]:
            # Take the seeding from the phase with *all* Event entrants
            if seed["phase"]["numSeeds"] == tournament.numEntrants:
                seeding = seed["groupSeedNum"]

        participant_data = standing["entrant"]["participants"][0]

        try:
            twitterHandle = participant_data \
                    ["player"] \
                    ["user"] \
                    ["authorizations"] \
                    [0] \
                    ["externalUsername"]
        except TypeError:
            twitterHandle = None

        if get_prefixes:
            prefix = participant_data["prefix"]
        else:
            prefix = ""

        player = smashgg.Player(
                id = standing["entrant"]["id"],
                prefix = prefix,
                gamerTag = participant_data["gamerTag"],
                placement = standing["placement"],
                seeding = seeding,
                twitterHandle = twitterHandle,
                )

        top_players[player.id] = player

    # -------------------------------------------------------------------------
    # Now, we need to find which characters those top players chose
    data = None

    data = smashgg.run_query(
            query_name = "getCharsByTournamentIdAndEntrantIds",
            variables = {
                "tournamentId" : int(tournament.id),
                "entrantIds": [ id for id in top_players.keys() ],
                },
            query_dir = ROOTDIR / "queries",
            token = token,
            proxy = proxy,
            log = log,
            )

    try:
        tournament_data = data["tournament"]
    except:
        log.error("Failed to load Tournament")
        return None,None

    if tournament_data is None:
        log.error("Failed to load Tournament")
        return None,None

    event = selectBiggestEvent(tournament_data, log)

    if event is None :
        return None,None

    # TODO check that sets number is < to hardcoded 100 max value (cf query)
    sets = event["sets"]["nodes"]

    for s in sets:
        try:
            for g in s["games"]:

                winnerId = g["winnerId"]

                for slct in g["selections"]:

                    if slct["selectionType"] == "CHARACTER":
                        eid = slct["entrant"]["id"]
                        try:
                            top_players[eid].add_character_selection(
                                    character = slct["selectionValue"],
                                    win = (winnerId == eid),
                                    )
                        except KeyError:
                            pass
        except TypeError:
            # If some games or selections are null, this can happen
            continue

    # Sort top_players by rank instead of id:
    top_players_sorted = sorted(
                top_players.values(),
                key = lambda p: p.placement,
                )

    # Return the data
    return tournament, top_players_sorted

# =============================================================================
if __name__ == '__main__':
    rv = main()
    sys.exit(rv)

